<?php
namespace BricksUltimate\Admin;

use BricksUltimate\{RegisterElements, Plugin};
use BricksUltimate\Admin\{License, TermsColumns, ProductSwatchesMetabox};

class Admin {

	private $menu_name = 'BricksUltimate';
	private $menu_slug = 'bricksultimate';
	private $buwl = '';

	function __construct() {
		$this->bu_save_whitelabel_data();

		$this->buwl = get_option('buwl');
		if( ! empty( $this->buwl ) ) {
			$this->menu_name = ! empty( $this->buwl['menu_name'] ) ? esc_html( $this->buwl['menu_name'] ) : $this->menu_name;
			$this->menu_slug = ! empty( $this->buwl['menu_slug'] ) ? esc_html( $this->buwl['menu_slug'] ) : $this->menu_slug;
		}

		add_action( 'admin_menu', array( $this, 'bu_register_admin_menu' ), 15 );

		add_filter( 'plugin_action_links', array( $this, 'bu_add_settings_link' ), 10, 2 );
		add_filter( 'network_admin_plugin_action_links', array( $this, 'bu_add_settings_link' ), 10, 2 );
		
		if( is_admin() ) 
		{
			new License();
			new TermsColumns();
			add_action('admin_footer', [ $this, 'bu_change_settings_ui'] );
		}

		if( class_exists('WooCommerce') && !empty( Plugin::$bu_settings['swatches'] ) && Plugin::$bu_settings['swatches'] == 'enabled' ) {
			new ProductSwatchesMetabox();
		}
	}

	/**
	 * Registering the sub menu link
	 */
	function bu_register_admin_menu() {
		add_submenu_page( 'bricks', $this->menu_name, $this->menu_name, 'manage_options', $this->menu_slug, array( $this, 'render_options_form' ) );
	}

	/**
	 * Rendering the options form
	 */
	function render_options_form() {
		$tab = isset( $_GET['tab'] ) ? sanitize_text_field( $_GET['tab'] ) : false;
		
		if ( ! License::has_valid_license() ) {
			$tab = 'license';
		}

		$user_id 	= get_current_user_id();
		$permission = [ $user_id ];
		$permission = ! empty( $this->buwl['tab_permission'] ) ? explode( ",", $this->buwl['tab_permission'] ) : $permission;

		if ( isset( $_POST[ 'action' ] ) && $_POST[ 'action' ] == "save_misc_data" ) {
			if( isset( $_POST[ 'misc' ] ) ) {
				update_option('bu_settings', $_POST[ 'misc' ] );
			} else {
				delete_option('bu_settings');
			}
		}

		$settings = get_option('bu_settings');

		$plugin_name = empty( $this->buwl['plugin_name'] ) ? Plugin::instance()->get_plugin_data('Name') : esc_html( $this->buwl['plugin_name'] );
	?>
		<h1 class="addon-title"><?php echo $plugin_name; ?> <sup>v<?php echo Plugin::instance()->get_version(); ?></sup></h1>
		<div class="wrap">
			<h2 class="nav-tab-wrapper">
				<a href="?page=<?php echo $this->menu_slug; ?>&amp;tab=elements" class="nav-tab<?php echo ( $tab === false || $tab == 'editor' || $tab == 'elements' ) ? ' nav-tab-active' : '';?>"><span class="dashicons dashicons-admin-tools" style="height: 18px; font-size: 17px; margin-right: 3px;"></span> <?php _e( 'General', 'bricksultimate' ); ?></a>

				<?php if( class_exists('WooCommerce') ): ?>
					<a href="?page=<?php echo $this->menu_slug; ?>&amp;tab=woo" class="nav-tab<?php echo ( $tab === 'woo' ) ? ' nav-tab-active' : '';?>">
						<span class="dashicons dashicons-wordpress" style="height: 18px; font-size: 17px; margin-right: 3px;"></span> <?php _e( 'Woo', 'bricksultimate' ); ?></a>
				<?php endif; ?>

				<a href="?page=<?php echo $this->menu_slug; ?>&amp;tab=misc" class="nav-tab<?php echo ( $tab == 'misc' ) ? ' nav-tab-active' : '';?>"><span class="dashicons dashicons-admin-generic" style="height: 18px; font-size: 17px; margin-right: 3px;"></span> <?php _e( 'Misc', 'bricksultimate' ); ?></a>

				<?php if( ( in_array( (get_userdata( $user_id ) )->roles[0], $permission ) || in_array( $user_id, $permission ) ) && ( empty( $settings['wl_tab'] ) || $settings['wl_tab'] !== 'enabled' ) ) : ?>
					<a href="?page=<?php echo $this->menu_slug; ?>&amp;tab=whitelabel" class="nav-tab<?php echo ($tab == 'whitelabel') ? ' nav-tab-active' : '';?>">
						<span class="dashicons dashicons-edit" style="height: 18px; font-size: 17px; margin-right: 3px;"></span> <?php _e( 'White Label', 'bricksultimate' ); ?></a>
				<?php endif; ?>
				
				<a href="?page=<?php echo $this->menu_slug; ?>&amp;tab=license" class="nav-tab<?php echo ($tab == 'license') ? ' nav-tab-active' : '';?>">
					<span class="dashicons dashicons-admin-network" style="height: 18px; font-size: 17px; margin-right: 3px;"></span> <?php _e( 'License', 'bricksultimate' ); ?>
				</a>
			</h2>
	<?php
			if ( $tab === 'elements' || $tab == false ) {
				$this->bu_elements_wplist_table();
			}

			if ( $tab === 'woo' ) {
				$this->buwoo_elements_wplist_table();
			}

			if ( $tab === 'misc' ) { 
				$this->bu_misc_settings();
			}

			if ( $tab === 'whitelabel' && is_user_logged_in() && ( in_array( (get_userdata( $user_id ) )->roles[0], $permission ) || in_array( $user_id, $permission ) ) ) { 
				$this->bu_white_label();
			}

			if ( $tab === 'license' ) { 
				$this->bu_license_form();
			}

		echo '</div>';
	}

	/**
	 * Making the elements listing table
	 * 
	 * @return void
	 */
	private function bu_elements_wplist_table() {
		// check user capabilities
		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		}

		if( isset( $_POST['action'] ) && $_POST['action'] == "save_elements" ){
			$details = self::bu_slice_array( RegisterElements::get_elements() ); //, $_POST['cur_page']
			$active_els = (array) get_option('bu_active_els');

			if( isset( $_POST['buel'] ) ) {
				$posted_els = array_diff( $active_els, $_POST['buel'] );
				$prev_actv_els = array_diff($posted_els, array_keys( $details['elements'] ) );
				$prev_actv_els = array_merge( $prev_actv_els, $_POST['buel'] );

				if( empty( $prev_actv_els[0] ) ) { unset( $prev_actv_els[0] ); }

				update_option('bu_active_els', array_values( $prev_actv_els ) );
			} else {
				$prev_actv_els = array_diff($active_els, array_keys( $details['elements'] ) );
				if( count( $prev_actv_els ) - 1 <= 0 && empty( $prev_actv_els[0] ) )
					delete_option('bu_active_els');
				else
					update_option('bu_active_els', $prev_actv_els );
			}

			printf('<div class="notice notice-info is-dismissible"><p>%s</p></div>', __('Settings saved successfully.', 'bricksultimate'));
		} else {
			$details = self::bu_slice_array( RegisterElements::get_elements() );
		}

		$active_els = (array) get_option('bu_active_els');
		$url 		= add_query_arg( 'tab', 'elements', menu_page_url( $this->menu_slug, false ) );

		// variables for pagination links
		//$page_first = $details['page'] > 1 ? 1 : '';
		//$page_prev  = $details['page'] > 1 ? $details['page'] - 1 : '';
		//$page_next  = $details['page'] < $details['total_pages'] ? $details['page'] + 1 : '';
		//$page_last  = $details['page'] < $details['total_pages'] ? $details['total_pages'] : '';
		?>
		<h2 class="heading"><?php _e('General Elements', 'bricksultimate'); ?><br/>
			<span style="color: #e64be0; font-size: 10px;text-transform: uppercase;">
				<?php esc_html_e('Only active elements will show and use on the Bricks Builder', 'bricksultimate' ); ?>
			</span>
		</h2>

		<?php if( is_array( $active_els ) && ! empty( $active_els[0] ) ) { $count = count($active_els); ?>
			<p><strong>Activated: </strong><?php printf( translate_nooped_plural( _n_noop( '%s element', '%s elements', 'bricksultimate' ), $count, 'bricksultimate' ), number_format_i18n( $count ) ); ?></p>
		<?php } ?>
		<div class="form-plugin-links" style="display: flex;">
			<form method="post" action="<?php echo $url; ?>&paged=<?php //echo $details['page']; ?>" style="width: 100%;">
				<input type="hidden" name="action" value="save_elements" />
				<table class="wp-list-table widefat plugins">
					<thead>
						<tr>
							<td id="cb" class="manage-column column-cb check-column">
								<label class="screen-reader-text" for="cb-select-all-1">
									<?php esc_html_e('Activate All' ); ?></label>
									<input id="cb-select-all-1" type="checkbox">
							</td>
							<th scope="col" id="name" class="manage-column column-name column-primary">
								<?php esc_html_e('Enable All'); ?>
							</th>
							<th style="width: 12%; text-align: right;"><?php _e('Documentation'); ?></th>
						</tr>
					</thead>
					<tbody>
						<?php echo self::bu_elements_list( $details['elements'] ); ?>
					</tbody>
					<tfoot>
						<tr>
							<td id="cb" class="manage-column column-cb check-column">
								<label class="screen-reader-text" for="cb-select-all-1">
									<?php esc_html_e('Activate All' ); ?></label>
									<input id="cb-select-all-1" type="checkbox">
							</td>
							<th scope="col" id="name" class="manage-column column-name column-primary">
								<?php esc_html_e('Enable All'); ?>
							</th>
							<td></td>
						</tr>
					</tfoot>
				</table>

				<div class="tablenav bottom">
					<div class="alignleft actions bulkactions">
						<?php submit_button(); ?>
						<!--input type="hidden" name="cur_page" value="<?php //echo $details['page']; ?>"-->		
					</div>
					<!--div class="tablenav-pages">
						<span class="displaying-num"><?php echo $details['total_elements']; ?> <?php _e('items'); ?></span>
						<span class="pagination-links">
							<?php if( $page_first ) : ?>
								<a class="first-page button" href="<?php echo $url . '&paged=' . $page_first?>">«</a>
							<?php endif; ?>

							<?php if( $page_prev ) : ?>
								<a class="prev-page button" href="<?php echo $url . '&paged=' . $page_prev?>">‹</a>
							<?php endif; ?>

							<span class="screen-reader-text">Current Page</span>
							<span id="table-paging" class="paging-input">
								<span class="tablenav-paging-text"><?php echo $details['page']; ?> of <span class="total-pages"><?php echo $details['total_pages']; ?></span></span>
							</span>

							<?php if( $page_next ) : ?>
							<a class="next-page button" href="<?php echo $url . '&paged=' . $page_next?>"><span class="screen-reader-text">Next page</span><span aria-hidden="true">›</span></a>
							<?php endif; ?>

							<?php if( $page_last ) : ?>
								<a class="last-page button" href="<?php echo $url . '&paged=' . $page_last?>">»</a>
							<?php endif; ?>
						</span>
					</div-->
					<br class="clear">
				</div>

			</form>
		</div>
		<?php
	}

	/**
	 * Making the Woo elements listing table
	 * 
	 * @return void
	 */
	private function buwoo_elements_wplist_table() {
		// check user capabilities
		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		}

		$details = self::bu_slice_array( RegisterElements::get_woo_elements() );

		if( isset( $_POST['action'] ) && $_POST['action'] == "save_woo_elements" )
		{
			$details = self::bu_slice_array( RegisterElements::get_woo_elements() ); //, $_POST['cur_page']
			$active_woo_els = (array) get_option('bu_active_wcels');

			if( isset( $_POST['buwooel'] ) ) {
				$posted_els = array_diff( $active_woo_els, $_POST['buwooel'] );
				$prev_actv_els = array_diff($posted_els, array_keys( $details['elements'] ) );

				if( count( $prev_actv_els ) - 1 <= 0 && empty( $prev_actv_els[0] ) ) { array_pop( $prev_actv_els ); }

				update_option('bu_active_wcels', array_merge( $prev_actv_els, $_POST['buwooel'] ) );
			} else {
				$prev_actv_els = array_diff($active_woo_els, array_keys( $details['elements'] ) );

				if( count( $prev_actv_els ) - 1 <= 0 && empty( $prev_actv_els[0] ) )
					delete_option('bu_active_wcels');
				else
					update_option('bu_active_wcels', $prev_actv_els );
			}

			printf('<div class="notice notice-info is-dismissible"><p>%s</p></div>', __('Selected elements have been activated successfully.', 'bricksultimate'));
		}

		$active_woo_els = (array) get_option('bu_active_wcels');

		$url = add_query_arg( 'tab', 'woo', menu_page_url( $this->menu_slug, false ) );
		// variables for pagination links
		/*$page_first = $details['page'] > 1 ? 1 : '';
		$page_prev  = $details['page'] > 1 ? $details['page'] - 1 : '';
		$page_next  = $details['page'] < $details['total_pages'] ? $details['page'] + 1 : '';
		$page_last  = $details['page'] < $details['total_pages'] ? $details['total_pages'] : '';*/
		?>
		
		<h2 class="heading"><?php _e('WooCommerce Elements', 'bricksultimate'); ?><br/>
			<span style="color: #e64be0; font-size: 10px;text-transform: uppercase;">
				<?php esc_html_e('Only active elements will show and use on the Bricks Builder', 'bricksultimate' ); ?>
			</span>
		</h2>

		<?php if( ! empty( $active_woo_els ) && ! empty( $active_woo_els[0] ) ) { $count = count($active_woo_els); ?>
			<p><strong>Activated: </strong><?php printf( translate_nooped_plural( _n_noop( '%s element', '%s elements', 'bricksultimate' ), $count, 'bricksultimate' ), number_format_i18n( $count ) ); ?></p>
		<?php } ?>

		<div class="form-plugin-links" style="display: flex;">
			<form method="post" action="<?php echo $url; ?>&paged=<?php echo $details['page']; ?>" style="width: 100%;">
				<input type="hidden" name="action" value="save_woo_elements" />
				<table class="wp-list-table widefat plugins">
					<thead>
						<tr>
							<td id="cb" class="manage-column column-cb check-column">
								<label class="screen-reader-text" for="cb-select-all-1">
									<?php esc_html_e('Activate All' ); ?></label>
									<input id="cb-select-all-1" type="checkbox">
							</td>
							<th scope="col" id="name" class="manage-column column-name column-primary">
								<?php esc_html_e('Enable All'); ?>
							</th>
							<th style="width: 12%; text-align: right;"><?php _e('Documentation'); ?></th>
						</tr>
					</thead>
					<tbody>
						<?php echo self::buwoo_elements_list(); ?>
					</tbody>
					<tfoot>
						<tr>
							<td id="cb" class="manage-column column-cb check-column">
								<label class="screen-reader-text" for="cb-select-all-1">
									<?php esc_html_e('Activate All' ); ?></label>
									<input id="cb-select-all-1" type="checkbox">
							</td>
							<th scope="col" id="name" class="manage-column column-name column-primary">
								<?php esc_html_e('Enable All'); ?>
							</th>
							<td></td>
						</tr>
					</tfoot>
				</table>
				
				<div class="tablenav bottom">
					<div class="alignleft actions bulkactions">
						<?php submit_button(); ?>
						<input type="hidden" name="cur_page" value="<?php echo $details['page']; ?>">		
					</div>
					<!--div class="tablenav-pages">
						<span class="displaying-num"><?php echo $details['total_elements']; ?> <?php _e('items'); ?></span>
						<span class="pagination-links">
							<?php if( $page_first ) : ?>
								<a class="first-page button" href="<?php echo $url . '&paged=' . $page_first?>">«</a>
							<?php endif; ?>

							<?php if( $page_prev ) : ?>
								<a class="prev-page button" href="<?php echo $url . '&paged=' . $page_prev?>">‹</a>
							<?php endif; ?>

							<span class="screen-reader-text">Current Page</span>
							<span id="table-paging" class="paging-input">
								<span class="tablenav-paging-text"><?php echo $details['page']; ?> of <span class="total-pages"><?php echo $details['total_pages']; ?></span></span>
							</span>

							<?php if( $page_next ) : ?>
							<a class="next-page button" href="<?php echo $url . '&paged=' . $page_next?>"><span class="screen-reader-text">Next page</span><span aria-hidden="true">›</span></a>
							<?php endif; ?>

							<?php if( $page_last ) : ?>
								<a class="last-page button" href="<?php echo $url . '&paged=' . $page_last?>">»</a>
							<?php endif; ?>
						</span>
					</div-->
					<br class="clear">
				</div>

			</form>
		</div>
		<?php
	}

	/**
	 * Making the body part of the wp list table
	 */
	private static function buwoo_elements_list() {
		$elements = RegisterElements::get_woo_elements();
		$active_els = (array) get_option('bu_active_wcels');
		foreach ( $elements as $key => $element ) {
			if( isset( $element['show'] ) && ! $element['show'] )
				continue;
		?>
			<tr valign="top" <?php echo in_array($key, $active_els) ? 'class="active"' : 'class="inactive"'; ?>>
				<th scope="row" class="check-column">
					<input id="<?php echo $key; ?>" name="buwooel[]" type="checkbox" value="<?php echo $key; ?>" <?php echo in_array($key, $active_els) ? 'checked="checked"' : ''; ?> />
				</th>
				<td class="plugin-title column-primary">
					<?php echo '<strong>' . $element['name'] . '</strong>'; ?>
				</td>
				<td style="text-align: right;">
					<a href="<?php echo $element['link']; ?>" target="_blank"><?php _e('Read Doc'); ?></a>
				</td>
			</tr>
		<?php
		}
	}
	
	/**
	 * Registering the license activation/deactivation form
	 */
	private function bu_license_form() {
		$url = add_query_arg( 'tab', 'license', menu_page_url( $this->menu_slug, false ) );
		
		if ( isset( $_POST[ 'license_activate' ] ) ) {
			echo License::bu_acivate_license( $_POST[ 'bu_license_key' ] );
		}

		if ( isset( $_POST[ 'license_deactivate' ] ) ) {
			echo License::delete_license();
		}
		
		$status = get_option(License::API_KEY_STATUS_FIELD);

		?>
		<h2><?php _e( 'License Settings', 'bricksultimate' ); ?></h2>
		<p class="description">Add the license key you received when purchasing BricksUltimate add-on.</p>
		<form method="post" action="<?php echo $url; ?>" class="license-activate-form">
			<table class="form-table">
				<tbody>
					<tr valign="top">
						<th scope="row" valign="top">
							<?php _e( 'License Key' ); ?>
							<?php if ( $status !== false && $status == 'valid' ) { ?>
								<span style="color:green;">(<?php _e( 'active' ); ?>)</span>
							<?php } ?>
						</th>
						<td>
							<input id="license_key" name="bu_license_key" type="password" class="regular-text" placeholder="<?php _e( 'Enter your license key' ); ?>" value="<?php echo ( $status !== false && $status == 'valid' ) ? '*****************************' : '' ;?>" />
							<?php if ( $status !== false && $status == 'valid' ) { ?>
								<input type="submit" class="button-secondary button" name="license_deactivate" value="<?php _e( 'Deactivate License' ); ?>"/>
							<?php } else { ?>
								<input type="submit" class="button button-primary" name="license_activate" value="<?php _e( 'Activate License' ); ?>"/>
							<?php } ?>
						</td>
					</tr>
				</tbody>
			</table>
			<?php wp_nonce_field( 'bu_nonce_action', 'bu_nonce_field' ); ?>
		</form>
		<?php
	}

	private function bu_save_whitelabel_data() {
		// check user capabilities
		if ( ! current_user_can( 'manage_options' ) ) {
			return;
		}

		if( isset( $_POST['action'] ) && $_POST['action'] == "save_wl_data" ){
			if( isset( $_POST['buwl'] ) ) {
				update_option('buwl', $_POST['buwl']);
			} else {
				delete_option('buwl');
			}

			printf('<div class="notice notice-info is-dismissible"><p>%s</p></div>', __('Settings saved successfully.', 'bricksultimate'));

			$menu_slug = 'bricksultimate';
			$buwl = get_option('buwl');
			if( $buwl ) {
				$menu_slug = ! empty( $buwl['menu_slug'] ) ? esc_html( $buwl['menu_slug'] ) : $menu_slug;
			}

			$url = add_query_arg( 'tab', 'whitelabel', menu_page_url( $menu_slug, false ) ) . '&page='. $menu_slug;

			if ( wp_safe_redirect( $url ) ) {
			    exit;
			}
		}
	}

	private function bu_white_label() {
		$plugin_name 	= 'placeholder="BricksUltimate"';
		$plugin_uri 	= 'placeholder="https://bricksultimate.com"';
		$author_name 	= 'placeholder="Chinmoy Paul"';
		$author_uri 	= 'placeholder="https://paulchinmoy.com"';
		$menu_name 		= 'placeholder="BricksUltimate"';
		$menu_slug 		= 'placeholder="bricksultimate"';
		$menuslug 		= 'bricksultimate';
		$plugin_desc 	= '';
		$tab_permission = 'placeholder="Enter user ID. Use comma for multiple users"';

		if( $this->buwl ) {
			$plugin_name 	= ! empty( $this->buwl['plugin_name'] ) ? 'value="' . esc_html( $this->buwl['plugin_name'] ) . '"' : $plugin_name;
			$plugin_uri 	= ! empty( $this->buwl['plugin_uri'] ) ? 'value="' . esc_html( $this->buwl['plugin_uri'] ) . '"' : $plugin_uri;
			$author_name 	= ! empty( $this->buwl['author_name'] ) ? 'value="' . esc_html( $this->buwl['author_name'] ) . '"' : $author_name;
			$author_uri 	= ! empty( $this->buwl['author_uri'] ) ? 'value="' . esc_html( $this->buwl['author_uri'] ) . '"' : $author_uri;
			$plugin_desc 	= ! empty( $this->buwl['plugin_desc'] ) ? esc_html( $this->buwl['plugin_desc'] ) : $plugin_desc;
			$menu_name 		= ! empty( $this->buwl['menu_name'] ) ? 'value="' . esc_html( $this->buwl['menu_name'] ) . '"' : $menu_name;
			$menu_slug 		= ! empty( $this->buwl['menu_slug'] ) ? 'value="' . esc_html( $this->buwl['menu_slug'] ) . '"' : $menu_slug;
			$menuslug 		= ! empty( $this->buwl['menu_slug'] ) ? esc_html( $this->buwl['menu_slug'] ) : $menuslug;
			$tab_permission = ! empty( $this->buwl['tab_permission'] ) ? 'value="' . esc_html( $this->buwl['tab_permission'] ) . '"' : $tab_permission;
		}

		$url = add_query_arg( 'tab', 'whitelabel', menu_page_url( $menuslug, false ) );
	?>
		<h2><?php _e( 'White Label', 'bricksultimate' ); ?></h2>
		<p class="description">It gives you the ability to control and transform the appearance of the back-end.</p>
		<div style="border-top: 1px solid #ccd0d4; margin-top: 15px;">
			<form method="post" action="<?php echo $url; ?>">
				<table class="form-table">
					<tbody>
						<tr valign="top">
							<th scope="row" valign="top">
								<?php _e( 'Plugin Name', 'bricksultimate' ); ?>
							</th>
							<td>
								<input id="plugin_name" name="buwl[plugin_name]" type="text" class="regular-text" <?php echo $plugin_name; ?> />
							</td>
						</tr>

						<tr valign="top">
							<th scope="row" valign="top">
								<?php _e( 'Plugin URI', 'bricksultimate' ); ?>
							</th>
							<td>
								<input id="plugin_uri" name="buwl[plugin_uri]" type="url" class="regular-text" <?php echo $plugin_uri; ?> />
							</td>
						</tr>

						<tr valign="top">
							<th scope="row" valign="top">
								<?php _e( 'Author Name', 'bricksultimate' ); ?>
							</th>
							<td>
								<input id="author_name" name="buwl[author_name]" type="text" class="regular-text" <?php echo $author_name; ?> />
							</td>
						</tr>

						<tr valign="top">
							<th scope="row" valign="top">
								<?php _e( 'Author URI', 'bricksultimate' ); ?>
							</th>
							<td>
								<input id="author_uri" name="buwl[author_uri]" type="url" class="regular-text" <?php echo $author_uri; ?> />
							</td>
						</tr>

						<tr valign="top">
							<th scope="row" valign="top">
								<?php _e( 'Plugin Description', 'bricksultimate' ); ?>
							</th>
							<td>
								<textarea id="plugin_desc" name="buwl[plugin_desc]" class="large-text" cols="5" rows="8" ><?php echo $plugin_desc; ?></textarea>
							</td>
						</tr>

						<tr valign="top">
							<th scope="row" valign="top">
								<?php _e( 'Admin Menu Name', 'bricksultimate' ); ?><br/>
							</th>
							<td>
								<input id="menu_name" name="buwl[menu_name]" type="text" class="regular-text" <?php echo $menu_name; ?> />
							</td>
						</tr>
						<tr valign="top">
							<th scope="row" valign="top">
								<?php _e( 'Admin Menu Link Slug', 'bricksultimate' ); ?>
							</th>
							<td>
								<input id="menu_slug" name="buwl[menu_slug]" type="text" class="regular-text" <?php echo $menu_slug; ?> />
							</td>
						</tr>
						<tr valign="top">
							<th scope="row" valign="top">
								<?php _e( 'Permission', 'bricksultimate' ); ?><br/>
								<lebel style="font-weight: normal; color: #999;"><?php _e( 'who can access this page', 'bricksultimate' ); ?></lebel>
							</th>
							<td>
								<input id="tab_permission" name="buwl[tab_permission]" type="text" class="regular-text" <?php echo $tab_permission; ?> />
							</td>
						</tr>
					</tbody>
				</table>
				<?php wp_nonce_field( 'bu_nonce_action', 'bu_nonce_field' ); ?>
				<input type="hidden" name="action" value="save_wl_data" />
				<?php submit_button(); ?>
			</form>
		</div>
	<?php
	}

	/**
	 * Making the body part of the wp list table
	 */
	private static function bu_elements_list( $elements ) {
		$active_els = (array) get_option('bu_active_els');
		foreach ( $elements as $key => $element ) {
			if( isset( $element['show'] ) && ! $element['show'] )
				continue;
		?>
			<tr valign="top" <?php echo in_array($key, $active_els) ? 'class="active"' : 'class="inactive"'; ?>>
				<th scope="row" class="check-column">
					<input id="<?php echo $key; ?>" name="buel[]" type="checkbox" value="<?php echo $key; ?>" <?php echo in_array($key, $active_els) ? 'checked="checked"' : ''; ?> />
				</th>
				<td class="plugin-title column-primary">
					<?php echo '<strong>' . $element['name'] . '</strong>'; ?>
				</td>
				<td style="text-align: right;">
					<a href="<?php echo $element['link']; ?>" target="_blank"><?php _e('Read Doc'); ?></a>
				</td>
			</tr>
		<?php
		}
	}

	/**
	 * Slicing the array for pagination
	 */
	private static function bu_slice_array( $data = array(), $cur_page = 1 ) {
		$page = isset($_GET['paged']) ? intval($_GET['paged']) : $cur_page;
		$page = isset($_POST['cur_page']) ? intval($_POST['cur_page']) : $page;

		// The number of records to display per page
		$page_size = 15;

		foreach( $data as $key => $element ) {
			if( isset( $element['show'] ) && ! $element['show'] )
				unset( $data[ $key ] );
		}

		// Calculate total number of records, and total number of pages
		$total_records = count($data);
		$total_pages   = ceil($total_records / $page_size);

		// Validation: Page to display can not be greater than the total number of pages
		if ($page > $total_pages) {
		    $page = $total_pages;
		}

		// Validation: Page to display can not be less than 1
		if ($page < 1) {
		    $page = 1;
		}

		// Calculate the position of the first record of the page to display
		$offset = ($page - 1) * $page_size;

		// Get the subset of records to be displayed from the array
		//$data = array_slice($data, $offset, $page_size);

		return array( 'elements' => $data, 'page' => $page, 'total_pages' => $total_pages, 'total_elements' => $total_records );
	}

	/**
	 * Misc settings
	 */
	public function bu_misc_settings() {
		$url = add_query_arg( 'tab', 'misc', menu_page_url( $this->menu_slug, false ) );
		if ( isset( $_POST[ 'action' ] ) && $_POST[ 'action' ] == "save_misc_data" ) {
			printf('<div class="notice notice-info is-dismissible"><p>%s</p></div>', __('Settings saved successfully.', 'bricksultimate'));
		}

		$settings = get_option('bu_settings');
	?>
		<form method="post" action="<?php echo $url; ?>">
			<table id="tab-misc">
				<tbody>
					<tr>
						<th>
							<label for="adminbar">Show admin bar in the editor</label>
						</th>
						<td>
							<input type="checkbox" name="misc[admin_bar]" id="adminbar" value="enabled" <?php echo ( ! empty( $settings['admin_bar'] ) && $settings['admin_bar'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
							<label for="adminbar"><?php _e('Yes', 'bricksultimate'); ?></label>
						</td>
					</tr>

					<tr>
						<th>
							<label for="quicklinks">Quick Links Menu</label>
							<p class="description"><?php _e('Add a menu on the admin bar and you can easily jump to the builder editor.', 'bricksultimate'); ?></p>
						</th>
						<td>
							<input type="checkbox" name="misc[menu_bar]" id="quicklinks" value="enabled" <?php echo ( ! empty( $settings['menu_bar'] ) && $settings['menu_bar'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
							<label for="quicklinks"><?php _e('Enable quick links menu.', 'bricksultimate'); ?></label>
						</td>
					</tr>

					<tr>
						<th>
							<label for="whitelabel">Disable White Label Tab</label>
						</th>
						<td>
							<input type="checkbox" name="misc[wl_tab]" id="whitelabel" value="enabled" <?php echo ( ! empty( $settings['wl_tab'] ) && $settings['wl_tab'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
							<label for="whitelabel"><?php _e('Disable', 'bricksultimate'); ?></label>
						</td>
					</tr>

					<tr>
						<th>
							<label for="heading">Heading Element</label>
							<p class="description"><?php _e('Add the custom tag control with dynamic data option.', 'bricksultimate'); ?></p>
						</th>
						<td>
							<input type="checkbox" name="misc[heading]" id="heading" value="enabled" <?php echo ( ! empty( $settings['heading'] ) && $settings['heading'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
							<label for="heading"><?php _e('Enable to adding the custom tag control', 'bricksultimate'); ?></label>
						</td>
					</tr>

					<tr>
						<th>
							<label for="postcomments">Single Post Comments Element</label>
							<p class="description"><?php _e('Improve the inbuilt "Comments" element which is showing on single post page.', 'bricksultimate'); ?></p>
						</th>
						<td>
							<input type="checkbox" name="misc[post_comments]" id="postcomments" value="enabled" <?php echo ( ! empty( $settings['post_comments'] ) && $settings['post_comments'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
							<label for="postcomments"><?php _e('Enable to enhance the existing controls', 'bricksultimate'); ?></label>
							<p class="description"><?php _e('It will add extra controls like hover effect, focus, width, height, padding, margin, etc.', 'bricksultimate'); ?></p>
						</td>
					</tr>

					<tr>
						<th>
							<label for="imageMask">Image Mask</label>
							<p class="description"><?php _e('Add 64 different kind of svg shapes to the Brick\'s in-built image element.', 'bricksultimate'); ?></p>
						</th>
						<td>
							<input type="checkbox" name="misc[image_mask]" id="image_mask" value="enabled" <?php echo ( ! empty( $settings['image_mask'] ) && $settings['image_mask'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
							<label for="image_mask"><?php _e('Enable mask controls.', 'bricksultimate'); ?></label>
						</td>
					</tr>

					<?php if( is_plugin_active('wp-grid-builder-bricks/wp-grid-builder-bricks.php') ) : ?>
						<tr>
						<th>
							<label>WP Grid Builder Facets Styler</label>
							<p class="description"><?php _e('Adding custom styler for the filter types.', 'bricksultimate'); ?></p>
						</th>

						<td>
							<div class="setting-wrapper">
								<input type="checkbox" name="misc[gbl_wpgb_facet]" id="gbl_wpgb_facet" value="enabled" <?php echo ( ! empty( $settings['gbl_wpgb_facet'] ) && $settings['gbl_wpgb_facet'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
								<label for="gbl_wpgb_facet"><?php _e('Enable global styler(coming soon)', 'bricksultimate'); ?></label>
								<p class="description" style="margin-bottom: 12px;"><?php _e('Navigate to Builder Editor -> Settings -> Theme Styles -> Unlimate - WPGB Facets Styler.', 'bricksultimate'); ?></p>
							</div>
							<div class="setting-wrapper">
								<input type="checkbox" name="misc[wpgb_facet]" id="wpgb_facet" value="enabled" <?php echo ( ! empty( $settings['wpgb_facet'] ) && $settings['wpgb_facet'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
								<label for="wpgb_facet"><?php _e('Enable individual element styler', 'bricksultimate'); ?></label>
								<p class="description"><?php _e('It will extend the flexibility in the builder editor.', 'bricksultimate'); ?></p>
							</div>
						</td>
					</tr>

					<?php endif; ?>

					<!--tr>
						<th>
							<label>Menu Items Controls</label>
							<p class="description"><?php _e('You can make the mega menu for WP Menu items.', 'bricksultimate'); ?></p>
						</th>

						<td>
							<div class="setting-wrapper">
								<input type="checkbox" name="misc[mcutf]" id="mcutf" value="enabled" <?php //echo ( ! empty( $settings['mcutf'] ) && $settings['mcutf'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
								<label for="mcutf"><?php //_e('Enable', 'bricksultimate'); ?></label>
								<p class="description"><?php //_e('It will create the bridge between WP Menu and Nav nestable element.', 'bricksultimate'); ?></p>
							</div>
						</td>
					</tr-->

					<tr>
						<th>
							<label>Ultimate Query Builder</label>
							<p class="description"><?php _e('Add custom query providers for Query Loop builder.', 'bricksultimate'); ?></p>
						</th>

						<td>
							<div class="setting-wrapper">
								<input type="checkbox" name="misc[ultimate_query]" id="ultimate_query" value="enabled" <?php echo ( ! empty( $settings['ultimate_query'] ) && $settings['ultimate_query'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
								<label for="ultimate_query"><?php _e('Enable custom query providers.', 'bricksultimate'); ?></label>
								<p class="description"><?php _e('It will extend the default query loop builder options and add the flexibility in the builder editor.', 'bricksultimate'); ?></p>
							</div>
						</td>
					</tr>

					<?php if( class_exists('WooCommerce') ) : ?>
					<tr>
						<th>
							<label>Ultimate Woo Query Builder</label>
							<p class="description"><?php _e('best selling, top rated, on-sale, featured, related, etc products list will build by container query loop.', 'bricksultimate'); ?></p>
						</th>

						<td>
							<div class="setting-wrapper">
								<input type="checkbox" name="misc[woo_query]" id="woo_query" value="enabled" <?php echo ( ! empty( $settings['woo_query'] ) && $settings['woo_query'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
								<label for="woo_query"><?php _e('Make products list with custom query loop.', 'bricksultimate'); ?></label>
								<p class="description"><?php _e('It will extend the default query loop options and add the flexibility in the builder editor.', 'bricksultimate'); ?></p>
							</div>
						</td>
					</tr>

					<tr>
						<th>
							<label>Add To Cart Button</label>
						</th>

						<td>
							<div class="setting-wrapper">
								<input type="checkbox" name="misc[product_atc]" id="product_atc" value="enabled" <?php echo ( ! empty( $settings['product_atc'] ) && $settings['product_atc'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
								<label for="product_atc"><?php _e('Enable', 'bricksultimate'); ?></label>
								<p class="description"><?php _e('It will the styler option for view cart button.', 'bricksultimate'); ?></p>
							</div>
						</td>
					</tr>

					<tr>
						<th>
							<label>Free Shipping Notice</label>
						</th>

						<td>
							<div class="setting-wrapper">
								<input type="checkbox" name="misc[fsn]" id="fsn" value="enabled" <?php echo ( ! empty( $settings['fsn'] ) && $settings['fsn'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
								<label for="fsn"><?php _e('Enable', 'bricksultimate'); ?></label>
							</div>
						</td>
					</tr>

					<tr>
						<th>
							<label>Cart Content Element</label>
							<p class="description"><?php _e('Adjust the parameters of cart content layout.', 'bricksultimate'); ?></p>
						</th>

						<td>
							<!--div class="setting-wrapper">
								<input type="checkbox" name="misc[ccqty]" id="ccqty" value="disabled" <?php //echo ( ! empty( $settings['ccqty'] ) && $settings['ccqty'] == 'disabled' ) ? 'checked="checked"' : ''; ?>>
								<label for="ccqty"><?php //_e('Disable quantity field', 'bricksultimate'); ?></label>
							</div-->

							<div class="setting-wrapper">
								<input type="checkbox" name="misc[ccqtydd]" id="ccqtydd" value="enabled" <?php echo ( ! empty( $settings['ccqtydd'] ) && $settings['ccqtydd'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
								<label for="ccqtydd"><?php _e('Dropdown quantity field(coming soon)', 'bricksultimate'); ?></label>
							</div>

							<!--div class="setting-wrapper">
								<select name="misc[cctpl]" id="cctpl">
									<option value="none"><?php //esc_html_e( 'Select template', 'bricks' ); ?></option>
									<?php
										/*$templates = \Bricks\Templates::get_templates_list('section');
										foreach ( $templates as $templateId => $templateName ) {
											$seleted = ! empty( $settings['cctpl'] ) ? $settings['cctpl'] : 'none';
											echo '<option value="' . $templateId . '" ' . selected( $templateId, $seleted, false ) . '>' . $templateName . '</option>';
										}*/
									?>
								</select> 
								<label for="cctpl"><?php //_e('Add section below the cart items', 'bricksultimate'); ?></label>
							</div-->

							<div class="setting-wrapper"> 
								<select name="misc[ccempty]" id="ccempty">
									<option value="none"><?php esc_html_e( 'Select template', 'bricks' ); ?></option>
									<?php
										$templates = \Bricks\Templates::get_templates_list('section');
										foreach ( $templates as $templateId => $templateName ) {
											$seleted = ! empty( $settings['ccempty'] ) ? $settings['ccempty'] : 'none';
											echo '<option value="' . $templateId . '" ' . selected( $templateId, $seleted, false ) . '>' . $templateName . '</option>';
										}
									?>
								</select> 
								<label for="ccempty"><?php _e('Empty cart message', 'bricksultimate'); ?></label>
							</div>
						</td>
					</tr>

					<tr>
						<th>
							<label>Product Element</label>
							<p class="description"><?php _e('Add extra style options.', 'bricksultimate'); ?></p>
						</th>

						<td>
							<div class="setting-wrapper">
								<input type="checkbox" name="misc[product_el]" id="product_el" value="enabled" <?php echo ( ! empty( $settings['product_el'] ) && $settings['product_el'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
								<label for="product_el"><?php _e('Enable to enhance the controls.', 'bricksultimate'); ?></label>
								<p class="description"><?php _e('It will add more style options in the builder editor.', 'bricksultimate'); ?></p>
							</div>
						</td>
					</tr>

					<tr>
						<th>
							<label>Product Filter Element</label>
							<p class="description"><?php _e('Add extra style options.', 'bricksultimate'); ?></p>
						</th>

						<td>
							<div class="setting-wrapper">
								<input type="checkbox" name="misc[product_filter]" id="product_filter" value="enabled" <?php echo ( ! empty( $settings['product_filter'] ) && $settings['product_filter'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
								<label for="product_filter"><?php _e('Enable to enhance the controls.', 'bricksultimate'); ?></label>
								<p class="description"><?php _e('It will extend the contorl options in the builder editor.', 'bricksultimate'); ?></p>
							</div>
						</td>
					</tr>

					<tr>
						<th>
							<label>Related Products Element</label>
							<p class="description"><?php _e('Add extra style options.', 'bricksultimate'); ?></p>
						</th>

						<td>
							<div class="setting-wrapper">
								<input type="checkbox" name="misc[related_prds]" id="related_prds" value="enabled" <?php echo ( ! empty( $settings['related_prds'] ) && $settings['related_prds'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
								<label for="related_prds"><?php _e('Enable to enhance the controls.', 'bricksultimate'); ?></label>
								<p class="description"><?php _e('It will extend the contorl options in the builder editor.', 'bricksultimate'); ?></p>
							</div>
						</td>
					</tr>

					<tr>
						<th>
							<label>Product Up/Cross Sells Element</label>
							<p class="description"><?php _e('Add extra style options.', 'bricksultimate'); ?></p>
						</th>

						<td>
							<div class="setting-wrapper">
								<input type="checkbox" name="misc[upcross_sells]" id="upcross_sells" value="enabled" <?php echo ( ! empty( $settings['upcross_sells'] ) && $settings['upcross_sells'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
								<label for="upcross_sells"><?php _e('Enable to enhance the controls.', 'bricksultimate'); ?></label>
								<p class="description"><?php _e('It will extend the contorl options in the builder editor.', 'bricksultimate'); ?></p>
							</div>
						</td>
					</tr>

					<tr>
						<th>
							<label>Cart Items & Totals Element</label>
							<p class="description"><?php _e('Add extra style options.', 'bricksultimate'); ?></p>
						</th>

						<td>
							<div class="setting-wrapper">
								<input type="checkbox" name="misc[cart_items]" id="cart_items" value="enabled" <?php echo ( ! empty( $settings['cart_items'] ) && $settings['cart_items'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
								<label for="cart_items"><?php _e('Enable to enhance the controls.', 'bricksultimate'); ?></label>
								<p class="description"><?php _e('It will extend the contorl options in the builder editor.', 'bricksultimate'); ?></p>
							</div>
						</td>
					</tr>

					<tr>
						<th>
							<label>Wishlist cookie life</label>
						</th>
						<td>
							<div class="setting-wrapper">
								<?php $seleted = ! empty( $settings['wsl_cookie_life'] ) ? $settings['wsl_cookie_life'] : 'week'; ?>
								<select name="misc[wsl_cookie_life]" id="wsl_cookie_life">
									<option value="week" <?php echo selected( "week", $seleted, false ); ?>><?php esc_html_e( '1 week', 'bricksultimate' ); ?></option>
									<option value="month" <?php echo selected( "month", $seleted, false ); ?>><?php esc_html_e( '1 month', 'bricksultimate' ); ?></option>
									<option value="3months" <?php echo selected( "3months", $seleted, false ); ?>><?php esc_html_e( '3 months', 'bricksultimate' ); ?></option>
									<option value="year" <?php selected( "year", $seleted, false ); ?>><?php esc_html_e( '1 year', 'bricksultimate' ); ?></option>
								</select>
							</div>
						</td>
					</tr>

					<tr>
						<th>
							<label>Compare cookie life</label>
						</th>
						<td>
							<div class="setting-wrapper">
								<?php $seleted = ! empty( $settings['comp_cookie_life'] ) ? $settings['comp_cookie_life'] : 'week'; ?>
								<select name="misc[comp_cookie_life]" id="comp_cookie_life">
									<option value="week" <?php echo selected( "week", $seleted, false ); ?>><?php esc_html_e( '1 week', 'bricksultimate' ); ?></option>
									<option value="month" <?php echo selected( "month", $seleted, false ); ?>><?php esc_html_e( '1 month', 'bricksultimate' ); ?></option>
									<option value="3months" <?php echo selected( "3months", $seleted, false ); ?>><?php esc_html_e( '3 months', 'bricksultimate' ); ?></option>
									<option value="year" <?php echo selected( "year", $seleted, false ); ?>><?php esc_html_e( '1 year', 'bricksultimate' ); ?></option>
								</select>
							</div>
						</td>
					</tr>

					<tr>
						<th>
							<label>Add to cart notification</label>
						</th>
						<td>
							<div class="setting-wrapper">
								<?php $seleted = ! empty( $settings['atc_notice'] ) ? $settings['atc_notice'] : 'week'; ?>
								<select name="misc[atc_notice]" id="atc_notice">
									<option value="none"><?php esc_html_e( 'Select template', 'bricks' ); ?></option>
									<?php
										$templates = \Bricks\Templates::get_templates_list('section');
										foreach ( $templates as $templateId => $templateName ) {
											$seleted = ! empty( $settings['atc_notice'] ) ? $settings['atc_notice'] : 'none';
											echo '<option value="' . $templateId . '" ' . selected( $templateId, $seleted, false ) . '>' . $templateName . '</option>';
										}
									?>
								</select>
							</div>
						</td>
					</tr>

					<tr>
						<th>
							<label>Swatches</label>
							<p class="description"><?php _e('Single product page and quick add form', 'bricksultimate'); ?></p>
						</th>
						<td>
							<div class="setting-wrapper">
								<input type="checkbox" name="misc[swatches]" id="swatches" value="enabled" <?php echo ( ! empty( $settings['swatches'] ) && $settings['swatches'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
								<label for="swatches"><?php esc_html_e('Enable', 'bricksultimate'); ?></label><br/>
								<input type="checkbox" name="misc[sw_variation_images]" id="sw_variation_images" value="enabled" <?php echo ( ! empty( $settings['sw_variation_images'] ) && $settings['sw_variation_images'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
								<label for="sw_variation_images"><?php esc_html_e('Fetch images from product variations', 'bricksultimate'); ?></label>
							</div>
						</td>
					</tr>

					<tr>
						<th>
							<label>Checkout</label>
						</th>
						<td>
							<div class="setting-wrapper">
								<input type="checkbox" name="misc[couponform]" id="couponform" value="enabled" <?php echo ( ! empty( $settings['couponform'] ) && $settings['couponform'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
								<label for="couponform"><?php esc_html_e('Remove default coupon form', 'bricksultimate'); ?></label><br/>
								<input type="checkbox" name="misc[wc_payment]" id="wc_payment" value="enabled" <?php echo ( ! empty( $settings['wc_payment'] ) && $settings['wc_payment'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
								<label for="wc_payment"><?php esc_html_e('Disable payment gateway', 'bricksultimate'); ?></label>
								<div class="checkout-fields-wrapper">
									<div class="billing-fields">
										<h4><?php _e('Removes billing fields', 'bricksultimate'); ?></h4>
										<?php
											$billing_fields = WC()->checkout()->get_checkout_fields( 'billing' );
											foreach( $billing_fields as $key => $billing_field ) {
												if( empty( $billing_field['label'] ) )
													continue;
										?>	
										<input type="checkbox" name="misc[<?php echo $key; ?>]" id="<?php echo $key; ?>" value="enabled" <?php echo ( ! empty( $settings[ $key ] ) && $settings[ $key ] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
										<label for="<?php echo $key; ?>"><?php echo wp_kses_post( $billing_field['label'] ); ?></label><br/>
										<?php } ?>
									</div>
									<div class="shipping-fields">
										<h4><?php _e('Removes shipping fields', 'bricksultimate'); ?></h4>
										<?php
											$shipping_fields = WC()->checkout()->get_checkout_fields( 'shipping' );
											foreach( $shipping_fields as $key => $shipping_field ) {
												if( empty( $shipping_field['label'] ) )
													continue;
										?>	
										<input type="checkbox" name="misc[<?php echo $key; ?>]" id="<?php echo $key; ?>" value="enabled" <?php echo ( ! empty( $settings[ $key ] ) && $settings[ $key ] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
										<label for="<?php echo $key; ?>"><?php echo wp_kses_post( $shipping_field['label'] ); ?></label><br/>
										<?php } ?>
									</div>
								</div>
							</div>
						</td>
					</tr>
					<?php endif; ?>

					<tr>
						<th>
							<label for="uninstalldata">Remove Data on Uninstall?</label>
						</th>
						<td>
							<input type="checkbox" name="misc[delete_data]" id="delete_data" value="enabled" <?php echo ( ! empty( $settings['delete_data'] ) && $settings['delete_data'] == 'enabled' ) ? 'checked="checked"' : ''; ?>>
							<label for="delete_data"><?php _e('Enable it if you would like BricksUltimate to completely remove all of its data when the plugin is deleted.', 'bricksultimate'); ?></label>
						</td>
					</tr>
				</tbody>
			</table>

			<?php wp_nonce_field( 'bu_nonce_action', 'bu_nonce_field' ); ?>
			<input type="hidden" name="action" value="save_misc_data" />
			<?php submit_button(); ?>
		</form>
	<?php
	}

	/**
	 * Adding the settings link
	 */
	public function bu_add_settings_link( $links, $file ) {
		if ( $file === 'bricksultimate/bricksultimate.php' && current_user_can( 'install_plugins' ) ) {
			if ( current_filter() === 'plugin_action_links' ) {
				$url = admin_url( 'admin.php?page=' . $this->menu_slug );
			} else {
				$url = admin_url( '/network/admin.php?page=' . $this->menu_slug );
			}

			$settings = sprintf( '<a href="%s">%s</a>', $url, __( 'Settings' ) );
			array_unshift(
				$links,
				$settings
			);
		}

		return $links;
	}

	function bu_change_settings_ui() {
		$screen = get_current_screen();

		if( empty( $screen ) || ! strstr( $screen->id, $this->menu_slug ) )
			return;

		$class = 'bricks_page_' . $this->menu_slug;

	    echo "<style type=\"text/css\">
	    	.{$class} .addon-title sup {
	    		color: #000;
				font-size: 10px;
				background: #ffdf0a;
				padding: 4px 8px;
				border-radius: 5px;
	    	}
	    	.{$class} .wrap {
				background: #fff;
				padding: 0 40px 40px;
				max-width: 100%;
				margin: 40px 15px 0 0;
				box-shadow: 0 0 18px #ddd;
			}

			.{$class} .wrap .heading{
				font-size: 1.8em;
			}

			.{$class} .nav-tab-wrapper, 
			.{$class} .wrap h2.nav-tab-wrapper {
				padding: 38px 35px 0;
				margin: 0 -40px 30px;
			}

			.{$class} .nav-tab {
				background-color: #fcfcfc;
				color: #777;
				display: flex; 
				align-items: center;
				padding: 10px 18px 6px;
				font-size: 12px;
			}

			.{$class} .about-wrap .nav-tab-active, 
			.{$class} .nav-tab-active, 
			.{$class} .nav-tab-active:hover {
				background-color: #fff;
				border-bottom-color: #fff;
				color: #333;
			}

			.{$class} table.widefat {
				border-left: 0;
				border-right: 0;
			}

			.{$class} .plugins .active td, 
			.{$class} .plugins .active th {
				background-color: #fafafa;
			}

			.{$class} .plugins .active th.check-column {
				border-left: 0;
			}

			.{$class} p.submit {
				padding: 0;
				margin: 0;
			}
			.{$class} .wp-list-table{
				margin-bottom: 10px;
			}
			.{$class} input[type=text],
			.{$class} input[type=email],
			.{$class} input[type=password],
			.{$class} input[type=url] {
				padding: 7px 12px;
			}
			.{$class} input[type=text],
			.{$class} input[type=email],
			.{$class} input[type=url],
			.{$class} input[type=password],
			.{$class} textarea {
				border: 1px solid #cdcdcd;
				color: #333;
				width: 100%;
			}
			.{$class} ::placeholder {
				color:#999;
			}
			.{$class} .license-activate-form {
				background-color: #fff;
				border: 1px solid #ccd0d4;
				padding: 5px 30px 15px;
			}
			.{$class} .license-activate-form .button {
				margin-top: 12px;
			}

			.{$class} #tab-misc {
				border-spacing: 0;
				margin: 0 0 30px;
				width: 100%;
			}

			.{$class} #tab-misc tbody {
				border-spacing: 0;
				display: flex;
				flex-direction: column;
				width: 100%;
			}

			.{$class} #tab-misc tr {
				background-color: #fff;
				box-shadow: 0 0 0 1px #ddd;
				display: flex;
			}

			.{$class} #tab-misc th {
				border-right: 1px solid #ddd;
				font-weight: 400;
				line-height: 1.7;
				max-width: 40%;
				padding: 15px 20px;
				text-align: initial;
				vertical-align: initial;
				width: 500px;
			}

			th label {
				cursor: default;
				font-size: 14px;
				font-weight: 700;
				display: inline-block;
				margin-right: 20px;
				min-width: 120px;
				text-transform: capitalize;
			}

			.{$class} #tab-misc td {
				background-color: #fff;
				padding: 20px 30px;
				text-align: initial;
				width: 100%;
			}

			.{$class} #tab-misc .description {
				color: #9e9e9e;
				font-size: 12px;
				line-height: 1.4;
				margin: 5px 0 0;
			}

			.{$class} #tab-misc input[type=checkbox] {
				appearance: none;
				background-color: #eaecef;
				border: none;
				border-radius: 16px;
				box-shadow: none;
				cursor: pointer;
				display: inline-block;
				height: 16px;
				line-height: 0;
				margin: 0 4px 0 0;
				outline: none;
				padding: 0;
				width: 26px;
				vertical-align: middle;
				transition: .05s border-color ease-in-out;
			}

			.{$class} #tab-misc input[type=checkbox]:checked {
				background-color: #2271b1;
			}

			.{$class} #tab-misc input[type=checkbox]:before {
				-webkit-appearance: none;
				background-color: #9da8b2;
				border-radius: 12px;
				content: '';
				cursor: pointer;
				display: block;
				height: 12px;
				left: 2px;
				margin: 0;
				position: relative;
				top: 2px;
				transition: .1s;
				transition: all .2s ease-out;
				width: 12px;
			}

			.{$class} #tab-misc input[type=checkbox]:checked:before {
				background-color: #fff;
				content: '';
				float: left;
				left: 0;
				opacity: 1;
				transform: translateX(100%);
				vertical-align: middle;
			}

			.{$class} .widefat th {
				font-weight: 700;
			}

			.{$class} select {
				font-size: 13px;
				margin-top: 5px;
			}

			.checkout-fields-wrapper {
				display: flex;
				margin-top: 20px;
				gap: 30px
			}

			.billing-fields,
			.shipping-fields {
				border: 1px solid #dedede;
				padding: 10px 30px 30px;
			}
	    </style>";
	}
}