<?php
namespace BricksUltimate;

if ( ! defined( 'ABSPATH' ) ) exit;

use BricksUltimate\Plugin;
use BricksUltimate\Admin\License;

class Compare {
	public static $instance = null;
	public static $compare_ids = [];
	public static $key = 'bu-compare';
	public static $inited = false;

	/**
	 * Saves compare item ids in cookie
	 */
	const COOKIE_KEY = 'bu_compare';

	public function __construct() {
		if ( License::has_valid_license() ) {
			add_action( 'wp_ajax_bu_update_compare_items', [ $this, 'update_compare_items' ] );
			add_action( 'wp_ajax_nopriv_bu_update_compare_items', [ $this, 'update_compare_items' ] );
			add_action( 'wp_ajax_bu_delete_compare_item', [ $this, 'delete_compare_item' ] );
			add_action( 'wp_ajax_nopriv_bu_delete_compare_item', [ $this, 'delete_compare_item' ] );

			$this->init_actions();
		}
	}

	public function update_compare_items() {
		check_ajax_referer( 'compare-nonce', 'security' );

		$data = ['success' => false];

		global $postData;

		$postData = [];
		$postId 	= $_POST['productId'];
		$postType 	= $_POST['postType'];
		$template_id = intval( $_POST['notice'] );
		$action 	= $_POST['compaction'];
		$cookie_key = $_POST['cookieKey'];
		$all_products = $this->get_compare_items( $cookie_key, $postType );
		self::$compare_ids[ $postType ] = isset( $all_products[ $postType ] ) ? $all_products[ $postType ] : [];

		if( ! in_array( $postId, self::$compare_ids[ $postType ] ) && $action == 'add' ) {
			$all_products[ $postType ][] = $postId;
			$data['success'] = true;
			$data['count'] = count( $all_products[ $postType ] );

			if( $template_id > 0 ) {
				$postData['id'] = $postId;
				$postData['filterType'] = 'notice';
				
				add_filter( 'bricks/posts/query_vars', [ '\BricksUltimate\Ajax', 'filter_query_args' ], 10, 3 );

				$elements = get_post_meta( $template_id, BRICKS_DB_PAGE_CONTENT, true );
				$template_inline_css = \Bricks\Templates::generate_inline_css( $template_id, $elements );

				$css = "<style id=\"bricks-inline-css-template-{$template_id}\">{$template_inline_css}</style>";

				$data['notice'] = do_shortcode('[bricks_template id="'. $template_id .'"]') . $css ;
				remove_filter( 'bricks/posts/query_vars', [ '\BricksUltimate\Ajax', 'filter_query_args' ], 10, 3 );
			}
		}

		if( in_array( $postId, self::$compare_ids[ $postType ] ) && $action == 'delete' ) {
			if( ( $key = array_search($postId, $all_products[ $postType ]) ) !== false )
				unset($all_products[ $postType ][$key]);

			$data['success'] = true;
			$data['count'] = ($all_products[ $postType ]) ? count( $all_products[ $postType ] ) : 0;
		}

		if( $data['success'] ) {
			Plugin::bu_setcookie($cookie_key, implode(',', $all_products[ $postType ]), time() + ($this->get_days_cache() * WEEK_IN_SECONDS) );

			if( is_user_logged_in() ) {
				update_user_meta( get_current_user_id(), $cookie_key, implode(',', $all_products[ $postType ]) );

				Plugin::bu_setcookie($cookie_key, '', time() + ($this->get_days_cache() * WEEK_IN_SECONDS) );
				unset( $_COOKIE[$cookie_key] );
			}

			self::$compare_ids[ $postType ] = $all_products[ $postType ];
		}

		wp_send_json( $data );

		wp_die();
	}

	public function delete_compare_item() {
		check_ajax_referer( 'compare-nonce', 'security' );

		$data = ['success' => true];

		$postId 	= $_POST['postId'];
		$postType 	= $_POST['postType'];
		$cookie_key = $this->get_cookie_key( $postType );
		$all_products = $this->get_compare_items( $cookie_key, $postType );
		self::$compare_ids[ $postType ] = isset( $all_products[ $postType ] ) ? $all_products[ $postType ] : [];

		if( in_array( $postId, self::$compare_ids[ $postType ] ) ) {
			if( ( $key = array_search($postId, $all_products[ $postType ]) ) !== false )
				unset($all_products[ $postType ][$key]);

			$data['success'] = true;
			$data['count'] = ($all_products[ $postType ]) ? count( $all_products[ $postType ] ) : 0;
		}

		if( $data['success'] ) {
			Plugin::bu_setcookie($cookie_key, implode(',', $all_products[ $postType ]), time() + ($this->get_days_cache() * WEEK_IN_SECONDS) );

			if( is_user_logged_in() ) {
				update_user_meta( get_current_user_id(), $cookie_key, implode(',', $all_products[ $postType ]) );

				Plugin::bu_setcookie($cookie_key, '', time() + ($this->get_days_cache() * WEEK_IN_SECONDS) );
				unset( $_COOKIE[$cookie_key] );
			}

			self::$compare_ids[ $postType ] = $all_products[ $postType ];
		}

		wp_send_json( $data );

		wp_die();
	}

	public function set_compare_items( $cookie_key = null, $type = 'post', $user_id = null ) {
		$compare_ids = [];

		$cached_products = '';
		$user_cached_products = false;

		if ( $user_id ) {
			$cached_products = get_user_meta( $user_id, $cookie_key, true );
			$user_cached_products = true;
		} elseif ( is_user_logged_in() ) {
			$cached_products = get_user_meta( get_current_user_id(), $cookie_key, true );
			$user_cached_products = true;
		}

		if ( !$user_cached_products && isset( $_COOKIE[ $cookie_key ] ) && !empty( $_COOKIE[ $cookie_key ] ) ) {
			$cached_products = $_COOKIE[ $cookie_key ];
		}

		if ( $cached_products != '' )
			$compare_ids[ $type ] = explode( ',', $cached_products);

		return $compare_ids;
	}

	public function get_compare_items($cookie_key = null, $post_type = 'post', $user_id = null) {
		if( $cookie_key ) {
			return $this->set_compare_items( $cookie_key, $post_type );
		} else {
			$postTypes = \Bricks\Helpers::get_registered_post_types();

			if( $postTypes ) {
				$ids = [];
				foreach( $postTypes as $type => $label ) {
					$cookie_key = $this->get_cookie_key( $type );
					if( isset( $_COOKIE[ $cookie_key ] ) && !empty( $_COOKIE[ $cookie_key ] ) ) {
						$ids[ $type ] = isset( $this->set_compare_items( $cookie_key, $type )[ $type ] ) ? $this->set_compare_items( $cookie_key, $type )[ $type ] : [];
					} elseif( is_user_logged_in() && metadata_exists( 'user', get_current_user_id(), $cookie_key )) {
						$ids[ $type ] = isset( $this->set_compare_items( $cookie_key, $type )[ $type ] ) ? $this->set_compare_items( $cookie_key, $type )[ $type ] : [];
					}
				}

				return $ids;
			} else {
				return $this->set_compare_items( $cookie_key, $post_type );
			}
		}
    }

    public function save_user_compare_items( $type = 'post', $user_id = null, $cookie_key = null, $compare_ids = [] ) {
    	$saved_products = get_user_meta($user_id, $cookie_key, true);

		if ( $saved_products ) {
			$saved_products = explode(',', $saved_products);
		} else {
			$saved_products = [];
		}

		$ready_products = array_unique( array_merge( $saved_products, $compare_ids ) );
		update_user_meta($user_id, $cookie_key, implode(',', $ready_products) );

		Plugin::bu_setcookie($cookie_key, '', time() + ($this->get_days_cache() * WEEK_IN_SECONDS));
		unset( $_COOKIE[$cookie_key] );
		self::$compare_ids[ $type ] = $ready_products;
    }

    public function update_compare_ids_after_login( $user_login, $user ){
		$items = $this->get_compare_items();
		$postTypes = \Bricks\Helpers::get_registered_post_types();

		if( $postTypes ) {
			foreach( $postTypes as $type => $label ) {
				$cookie_key = $this->get_cookie_key( $type );
				if( isset( $_COOKIE[ $cookie_key ] ) && ! empty( $_COOKIE[ $cookie_key ] ) ) {
					$this->save_user_compare_items( $type, $user->ID, $cookie_key, ( $items[$type] ?? [] ) );
				} elseif ( metadata_exists( 'user', $user->ID, $cookie_key ) ) {
					$this->save_user_compare_items( $type, $user->ID, $cookie_key, ( $items[$type] ?? [] ) );
				}
			}
		}
    }

    public static function get_days_cache() {
		$cache_days = 7;

		$life = ( !empty( Plugin::$bu_settings['comp_cookie_life'] ) ) ? Plugin::$bu_settings['comp_cookie_life'] : 'week';

		switch ( $life ) {
			case 'week':
				$cache_days = 7;
				break;
			case 'month':
				$cache_days = 30;
				break;
			case '3months':
				$cache_days = 30*3;
				break;
			case 'year':
				$cache_days = 365;
				break;
		}

		return $cache_days;
	}

	public static function get_compare_counter( $post_type ) {
	    return ! empty( self::$compare_ids[ $post_type ] ) ? count(self::$compare_ids[ $post_type ]) : 0;
    }

	public static function get_cookie_key($post_type = 'post' ){
		return self::COOKIE_KEY . "_{$post_type}_" . ( is_multisite() ? get_current_blog_id() : 0 );
	}

	public static function is_item_in_comparelist( $itemId, $post_type ) {
		return in_array($itemId, self::$compare_ids[ $post_type ]);
	}

	public function init_compare() {
		self::$compare_ids = $this->get_compare_items();
		self::$inited = true;
	}

	public function init_actions() {
		add_action( 'wp_login', [ $this, 'update_compare_ids_after_login' ], 10, 2);
		add_action( 'init', function () {
			if ( ! self::$inited ) {
				$this->init_compare();
			}
		});
	}

    public static function get_instance() {
        if ( null == self::$instance ) {
            self::$instance = new self();
        }

        return self::$instance;
    }
}